package strategy

import (
	"backea/internal/backup/models"
	"context"
	"io"
	"time"
)

// BackupInfo contains information about a single backup
type BackupInfo struct {
	ID           string    // Unique identifier for the backup
	CreationTime time.Time // When the backup was created
	Size         int64     // Size in bytes
	Source       string    // Original source path
	Type         string    // Type of backup (e.g., "kopia", "restic")
	RetentionTag string    // Retention policy applied (e.g., "latest", "daily", "weekly")
}

// StorageUsageInfo contains information about backup storage usage
type StorageUsageInfo struct {
	TotalBytes int64  // Total bytes stored
	Provider   string // Storage provider (e.g., "local", "b2", "s3")
	ProviderID string // Provider-specific ID (bucket name, path, etc.)
}

// Strategy defines the backup/restore strategy interface
type Strategy interface {
	// Execute performs the backup operation
	Execute(ctx context.Context, serviceName, directory string) error

	// ListBackups returns information about existing backups
	ListBackups(ctx context.Context, serviceName string) ([]models.BackupInfo, error)

	// GetStorageUsage returns information about storage usage
	GetStorageUsage(ctx context.Context, serviceName string) (*models.StorageUsageInfo, error)

	// RestoreBackup restores a backup by ID
	RestoreBackup(ctx context.Context, backupID string, serviceName string) error

	// DownloadBackup provides a reader with backup summary information
	DownloadBackup(ctx context.Context, backupID string, serviceName string) (io.ReadCloser, error)

	// GetBackupInfo returns detailed information about a specific backup
	GetBackupInfo(ctx context.Context, backupID string, serviceName string) (*models.BackupInfo, error)
}

// ServiceConfig represents the configuration for a service's backup
type BackupConfig struct {
	BackupStrategy StrategyConfig // Strategy to use for backups
}

// StrategyConfig defines configuration for a backup strategy
type StrategyConfig struct {
	Type        string            // Type of backup strategy (e.g., "kopia", "rsync")
	Provider    string            // Storage provider (e.g., "local", "b2", "sftp")
	Destination DestinationConfig // Destination configuration
	Retention   RetentionConfig   // Retention policy
}

// DestinationConfig defines backup destination settings
type DestinationConfig struct {
	Host     string // Host for remote destination
	Path     string // Path for storage
	Username string // Username for authentication
	SSHKey   string // SSH key for authentication
}

// RetentionConfig defines backup retention policy
type RetentionConfig struct {
	KeepLatest  int // Number of latest snapshots to keep
	KeepHourly  int // Number of hourly snapshots to keep
	KeepDaily   int // Number of daily snapshots to keep
	KeepWeekly  int // Number of weekly snapshots to keep
	KeepMonthly int // Number of monthly snapshots to keep
	KeepYearly  int // Number of yearly snapshots to keep
}
